//  Sur un Arduino avec ATmega328, on travaille en bus CAN via le bus SPI
//  Il faut donc le déclarer
#include <SPI.h>

//  Déclaration de la bubliothèque CAN pour ATmega328
#include <mcp_can.h>

const int SPI_CS_PIN = 10;               //  On n'a pas le choix de la broche : c'est la broche 10
MCP_CAN CAN(SPI_CS_PIN);                 //  déclaration de la broche Chip Select du MCP 2515

byte presence[4];                        //  Présence sur l'aiguille (1 parmi 4 de la carte)
boolean octet_position_aiguille[7];      //  Octet de position aiguille

int mesure[4];                           //  Mesure de la tension sur la broche
int nombre_mesures[4];                   //  Nombre de mesures
int tension_moyenne[4];                  //  Tension moyenne mesurée
int tension_finale[4];                   //  Tension finale suite aux mesures
int tension[4];                          //  Tension courante
boolean valid[4];                        //  Validation

byte data[8];                            //  Les données envoyées
byte ancienne_data[8];                   //  Anciennes données envoyées

byte buf[4];                             //  Les données reçues
byte ancien_buf[4];                      //  Anciennes données reçues

int  canId;                              //  Identifiant CAN (sur 11 bits)
byte len = 0;                            //  Nombre d(octets transmis

//-------------------------- Up_down --------------------------------
void up_down (byte ordre, byte pin_bouton)
{
    mesure[ordre]        = analogRead(ordre + pin_bouton);
    nombre_mesures[ordre] = nombre_mesures[ordre] + 1;
    
    if (((tension_moyenne[ordre] > 750) || (tension_moyenne[ordre] < 250)) && (nombre_mesures[ordre] > 2))
    {
        tension_finale[ordre]   = tension_moyenne[ordre];
        nombre_mesures[ordre] = 0;
        tension[ordre]        = 0;
        valid[ordre]          = true;
    }
    else
    {
        tension[ordre]        = tension[ordre] + mesure[ordre];
        tension_moyenne[ordre] = tension[ordre] / nombre_mesures[ordre];    
    }
}

//----------------------------------- SETUP ---------------------------------------------
void setup()
{
    Serial.begin(115200);
    
    //  pinMode(1,OUTPUT);  //  LED aiguille 1 vers la gauche. Servira plus tard
    //  La pin 2 est la pin d'interruption du MCP 2515 et gérée par la bibliothèque
    //  Donc, pas de déclaration dans le Setup
    pinMode(3,OUTPUT);      //  LED aiguille 1 vers la droite
    
    pinMode(4,OUTPUT);      //  LED aiguille 2 vers la gauche
    pinMode(5,OUTPUT);      //  LED aiguille 2 vers la droite
    
    pinMode(6,OUTPUT);      //  LED aiguille 3 vers la droite
    pinMode(7,OUTPUT);      //  LED aiguille 3 vers la gauche
    
    pinMode(8,OUTPUT);      //  LED aiguille 4 vers la gauche
    pinMode(9,OUTPUT);      //  LED aiguille 4 vers la droite
    
    pinMode(A0, OUTPUT);    //  LED présence aiguille 1
    pinMode(A1, OUTPUT);    //  LED présence aiguille 2
    pinMode(A2, OUTPUT);    //  LED présence aiguille 3
    pinMode(A3, OUTPUT);    //  LED présence aiguille 4

    //  A4 BP changement aiguille 1
    //  A5 BP changement aiguille 2
    //  A6 BP changement aiguille 3 (uniquement analogique)
    //  A7 BP changement aiguille 4 (uniquement analogique)   

START_INIT:

    if(CAN_OK == CAN.begin(CAN_250KBPS)) // init can bus : baudrate = 250k
    {
        Serial.println("CAN BUS Shield init ok!");
    }
    else
    {
        Serial.println("CAN BUS Shield init fail");
        Serial.println("Init CAN BUS Shield again");
        delay(100);
        goto START_INIT;
    }
}

//-------------------------------------- LOOP -------------------------------------------------
void loop()
{
    //------- Envoi des données changement de position au gestionnaire ---------------------    
    for (byte i = 0; i<4; i++)
    {
        valid[i] = false;  
        up_down(i,18);  //  Quand on sort d'up/down, on a une information fiable
        
        if (valid[i] = true)
        {        
            if (tension_finale[i] < 750)
            {
                octet_position_aiguille[i] = true;
            }
            else
            {
                octet_position_aiguille[i] = false;
            }          
        }            
    }

    //  On n'envoie que 4 BP pour le test
    //  On n'envoie que pour la carte 1 (qui a l'identifiant 0x221)
    //  On pourrait gerer 64 BP en créant une boucle for incluant les 8 octets
    //  Ici, on se limite à 4 bits du 1er octet
    
    data[0] = 1 * octet_position_aiguille[0]
            + 2 * octet_position_aiguille[1]
            + 4 * octet_position_aiguille[2]
            + 8 * octet_position_aiguille[3];      
    data[1] = 0;
    data[2] = 0;
    data[3] = 0;
    data[4] = 0;
    data[5] = 0;
    data[6] = 0;
    data[7] = 0;
    
    //  On n'envoie des infos dans le bus CAN que quand elles ont changé
    //  Ici on envoie 8 octets :
    //  Dans le premier, seuls les 4 bits de poids faible sont envoyés
    //  Les 7 autres octets sont vides
    
    if(data[0] != ancienne_data[0])
    {
        Serial.print("Ancienne data : ");
        Serial.print(ancienne_data[0],BIN);
        Serial.print("     Nouvelle data : ");
        Serial.println(data[0],BIN);
        
        ancienne_data[0] = data[0];
        //  send data:  id = 0x201, trame standard, 8 octets, data
        CAN.sendMsgBuf(0x201, 0, 8, data);
        
        Serial.print("  2   Ancienne data : ");
        Serial.print(ancienne_data[0],BIN);
        Serial.print("     Nouvelle data : ");
        Serial.println(data[0],BIN);
    }
    
    //---------------------------- Réception présence des aiguilles ------------------

    if(CAN_MSGAVAIL == CAN.checkReceive()) //  Teste l'arrivée de données sur le bus CAN
    {        
        CAN.readMsgBuf(&len, buf);         //  On extrait des données le nombre d'octet et les données elle mêmes
        
        canId = CAN.getCanId();            //  On extrait l'identifiant 11 bits
       
        //----------------------- Allumage des LED de présence -----------------------
        if(canId == 0x211)
        {
            if ((ancien_buf[0] != buf[0])
             || (ancien_buf[1] != buf[1])
             || (ancien_buf[2] != buf[2])
             || (ancien_buf[3] != buf[3]))
            {
                Serial.println("-----------------------------");
                Serial.println("get data from ID: ");
                Serial.println(canId,HEX);
                Serial.println(len);
    
                for(byte i = 0; i<len; i++)    // impression des données reçues
                {
                    Serial.print(buf[i]);
                    Serial.print("\t");
                    Serial.println(buf[i],BIN);
                    ancien_buf[i] = buf[i];
                }
                Serial.println();
                
                //---------------------- LED de positon --------------------------------------
                if (bitRead(buf[0],7) == 1) //  On ne cherche que le bit 7 (le plus à gauche)
                {
                    digitalWrite (8,HIGH);
                }
                else
                {
                    digitalWrite (8,LOW);
                }
                if (bitRead(buf[1],7) == 1)
                {
                    digitalWrite (7,HIGH);
                }
                else
                {
                    digitalWrite (7,LOW);
                }
                if (bitRead(buf[2],7) == 1)
                {
                    digitalWrite (4,HIGH);
                }
                else
                {
                    digitalWrite (4,LOW);
                }
                if (bitRead(buf[3],7) == 1)
                {
                    digitalWrite (1,HIGH);
                }
                else
                {
                    digitalWrite (1,LOW);
                }
                if (bitRead(buf[0],6) == 1)
                {
                    digitalWrite (9,HIGH);
                }
                else
                {
                    digitalWrite (9,LOW);
                }
                if (bitRead(buf[1],6) == 1)
                {
                    digitalWrite (6,HIGH);
                }
                else
                {
                    digitalWrite (6,LOW);
                }
                if (bitRead(buf[2],6) == 1)
                {
                    digitalWrite (5,HIGH);
                }
                else
                {
                    digitalWrite (5,LOW);
                }
                if (bitRead(buf[3],6) == 1)
                {
                    digitalWrite (3,HIGH);
                }
                else
                {
                    digitalWrite (3,LOW);
                }
                
                //----------------------------- LED de présence -------------------------------                 
                for(byte i = 0; i<4; i++)
                {  
                    if (bitRead(buf[i],5) == 1)  //  On ne cherche que le bit 5
                    {
                        digitalWrite (i+14,HIGH);
                    }
                    else
                    {
                        digitalWrite (i+14,LOW);
                    }
                }                    
            }
        }       
    }
}
